var _ = require('lodash') ;
var https = require('https') ;
var axios = require('axios') ;
var xmlParser = require('fast-xml-parser') ;
var jsonPathFinding = require('jsonpath') ;
var JsonParser = require('fast-xml-parser').j2xParser ;
var defaultOptions = require('./config').defaultOptions ;


const performHttpRequest = async (options) => {
  let response;
  try {
    response = await axios(options.axios);
  } catch (error) {
    throw error;
  }

  return response;
};


function SoapClient(options) {
  this.options = _.merge(defaultOptions, options);
}

SoapClient.prototype.setBasicAuthSecurity = function (username, password) {
  const token = Buffer.from(`${username}:${password}`, 'utf-8').toString('base64');
  this.options.axios.headers.Authorization = `Basic ${token}`;
};

SoapClient.prototype.disableSSL = function () {
  this.options.axios.httpsAgent = new https.Agent({ rejectUnauthorized: false });
};

SoapClient.prototype.enableSSL = function () {
  this.options.fastXmlParser.httpsAgent = new https.Agent({ rejectUnauthorized: true });
};

SoapClient.prototype.setArrayMode = function (value) {
  this.options.fastXmlParser.xmlParser.arrayMode = value;
};

SoapClient.prototype.setJsonPath = function (value) {
  this.options.jsonPath = value;
};

SoapClient.prototype.performRequest = async function (soapUrl, jsonData) {
  const  debug = this.options.debug;
  const  isRequestDataXML = this.options.isRequestDataXML;
  const  isResponseXML = this.options.isResponseXML;
  var httpResponse,convertedJson;

  this.options.axios.url = soapUrl;

  if(isRequestDataXML){
    this.options.axios.data = jsonData;
  }else {
    try {
      const jsonParser = new JsonParser(this.options.fastXmlParser.jsonParser);
      const xml = jsonParser.parse(jsonData);
      debug && console.log("--DEBUG-- XML request body => ",xml);
      this.options.axios.data = xml;
    } catch (error) {
      const parsingError = { message: 'Error when converting from json to xml', error };
      debug && console.log("--DEBUG-- parsing error => ",parsingError);
      throw parsingError;
    }
  }

  try {
    httpResponse = await performHttpRequest(this.options);
    debug && console.log("--DEBUG-- http response => ",httpResponse);
  } catch (error) {
    const axiosError = { message: 'Error when performing the http request', error };
    debug && console.log("--DEBUG-- axios error => ",axiosError);
    throw axiosError;
  }

  if(isResponseXML){
    return httpResponse.data;
  }else{
    try {
       convertedJson = xmlParser.parse(httpResponse.data, this.options.fastXmlParser.xmlParser);
       debug && console.log("--DEBUG-- converted json => ",convertedJson);
    } catch (error) {
      const parsingError = {message: 'Error when converting from xml to Json', error};
      debug && console.log("--DEBUG-- parsing error => ",parsingError);
      throw parsingError;
    }

  const jsonPath  = this.options.jsonPath;
  if (jsonPath){
    try {
      const targetJson = jsonPathFinding.value(convertedJson, jsonPath);
      debug && console.log("--DEBUG-- target json => ",targetJson);

      return targetJson;
    } catch (error) {
      const jsonPathError = { message: 'Error when finding the target object using jsonPath', error };
      debug && console.log("--DEBUG-- jsonPath error => ",jsonPathError);
      throw jsonPathError;
    }
  } else return convertedJson;

}

};

module.exports = SoapClient ;
